#!/usr/bin/env python

import rospy
import getch
import sys
import termios
import tty
import select
from geometry_msgs.msg import Twist
from sensor_msgs.msg import Range

def altitude_callback(msg):
    global current_altitude, first_altitude
    #print("Altura: ")
    if current_altitude == 0:
        first_altitude = msg.range
    current_altitude = msg.range
    #print("Sonar Values : ", current_altitude)
    #print("\n")

def get_char():
    # Save the current terminal settings
    old_settings = termios.tcgetattr(sys.stdin)

    try:
        # Configure the terminal for raw input
        tty.setraw(sys.stdin.fileno())

        # Wait until data is available on stdin
        rlist, _, _ = select.select([sys.stdin], [], [], 0.1)

        if rlist:
            # Read a single character from the keyboard
            ch = sys.stdin.read(1)
        else:
            ch = None
    finally:
        # Restore the original terminal settings
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, old_settings)

    return ch
        
def start():
    global current_altitude, first_altitude
    pub = rospy.Publisher('/cmd_vel', Twist, queue_size=10)
    rospy.Subscriber('/sonar_height', Range, altitude_callback)
    rate = rospy.Rate(10)
    vel_msg = Twist()
    vel_msg.linear.z = 0.0  # Altura inicial
    elevation_speed = 0.5  # Velocidad de elevación
    yaw_speed = 0.5  # Velocidad de giro
    first_altitude = 0
    current_altitude = 0
    print("start")
    while not rospy.is_shutdown():
        key = get_char()
        if key:
            if ord(key) == 27:
                break
            if ord(key) == ord("t"):
                while current_altitude < 2:
                    vel_msg.linear.z = elevation_speed
                    pub.publish(vel_msg)
            if ord(key) == ord("y"):
                while current_altitude > first_altitude:
                    vel_msg.linear.z = -elevation_speed
                    pub.publish(vel_msg)
            if ord(key) == ord("w"):
                vel_msg.linear.x = 0.5
            elif ord(key) == ord("s"):
                vel_msg.linear.x = -0.5
            elif ord(key) == ord("a"):
                vel_msg.linear.y = 0.5
            elif ord(key) == ord("d"):
                vel_msg.linear.y = -0.5
            elif ord(key) == ord("q"):
                vel_msg.angular.z = yaw_speed
            elif ord(key) == ord("e"):
                vel_msg.angular.z = -yaw_speed
            if ord(key) == ord(" "):
                vel_msg.linear.z = elevation_speed  # Subir el drone
            elif ord(key) == ord("c"):
                vel_msg.linear.z = -elevation_speed  # Bajar el drone
        else:
            vel_msg.linear.x = 0.0
            vel_msg.linear.y = 0.0
            vel_msg.linear.z = 0.0  # Mantener la altura actual si no se presiona ninguna tecla de elevación
            vel_msg.angular.z = 0.0  # Detener el giro si no se presiona ninguna tecla de giro

        pub.publish(vel_msg)
        rate.sleep()

try:
    rospy.init_node('conducir_drone', anonymous=False)
    start()
except rospy.ROSInterruptException:
    pass       



