#!/usr/bin/env python3
#  -*- coding: utf-8 -*-
# from tkinter import *
from tkinter import ttk
from geometry_msgs.msg import Twist, PoseStamped
from nav_msgs.msg import Odometry
import tkinter as tk
import math
import rospy
from sensor_msgs.msg import Range


root = tk.Tk(className="Controll drone")
root.title("Controll drone")

mainframe = ttk.Frame(root, padding ="3 3 12 12")
mainframe.grid(column=0, row=0, sticky=(tk.N,tk.W,tk.E,tk.S))
mainframe.columnconfigure(0,weight=1)

x_p = tk.StringVar()
y_p = tk.StringVar()
z_p = tk.StringVar()
z_o = tk.StringVar()
current_altitude = 0
first_altitude = 0

def quaterionToRads(data):
	x = data.pose.orientation.x
	y = data.pose.orientation.y
	z = data.pose.orientation.z
	w = data.pose.orientation.w

	t3 = 2.0 * (w * z + x * y)
	t4 = 1.0 - 2.0 * (y * y + z * z)
	yawZActual = math.atan2(t3, t4)
	if yawZActual < 0:
		yawZActual = 2*math.pi + yawZActual

	return yawZActual

#Callback de pose y orientacion simulador
def pose_callback(data):
    x_p.set("{0:.2f}".format(data.pose.pose.position.x))
    y_p.set("{0:.2f}".format(data.pose.pose.position.y))
    z_p.set("{0:.2f}".format(data.pose.pose.position.z))

def rot_callback(data):
    z_o.set("{0:.2f}".format( math.degrees(quaterionToRads(data)) ))

def altitude_callback(msg):
    global current_altitude, first_altitude
    #print("Altura: ")
    if current_altitude == 0:
        first_altitude = msg.range
    current_altitude = msg.range
    #print("Sonar Values : ", current_altitude)
    #print("\n")

rospy.init_node('HectorQ_GUI', anonymous=False)
#Subscribers
posicionLider_sub = rospy.Subscriber("/ground_truth/state", Odometry , pose_callback)
orientaLider_sub = rospy.Subscriber("/ground_truth_to_tf/pose", PoseStamped , rot_callback)
rospy.Subscriber('/sonar_height', Range, altitude_callback)

#Publishers
vel_pub = rospy.Publisher('/cmd_vel', Twist, queue_size=1)

def setText(text):
    ttk.Label(mainframe, text="              ").grid(column=3, row=1, sticky=tk.W)
    ttk.Label(mainframe, text=text).grid(column=3, row=1, sticky=tk.W)

def enviar_velocidad(vx,vy,vz,vaz):
    vel_msg = Twist()
    vel_msg.linear.x = float(vx)
    vel_msg.linear.y = float(vy)
    vel_msg.linear.z = float(vz)
    vel_msg.angular.z = float(vaz)
    vel_msg.angular.x = float(0.0)
    vel_msg.angular.y = float(0.0)
    vel_pub.publish(vel_msg)

def hover():
    setText("Hover")
    enviar_velocidad(0.0,0.0,0.0,0.0)

def takeoff():
    global current_altitude, first_altitude
    setText("TakeOff")
    hover()
    vel_msg = Twist()
    while current_altitude < 2:
        vel_msg.linear.z = 0.5
        vel_pub.publish(vel_msg)
    vel_msg.linear.x = 0.0
    vel_msg.linear.y = 0.0
    vel_msg.linear.z = 0.0  # Mantener la altura actual si no se presiona ninguna tecla de elevación
    vel_msg.angular.z = 0.0
    vel_pub.publish(vel_msg)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def land():
    global current_altitude, first_altitude
    setText("Land")
    hover()
    vel_msg = Twist()
    while current_altitude > first_altitude:
        vel_msg.linear.z = -0.5
        vel_pub.publish(vel_msg)
    vel_msg.linear.x = 0.0
    vel_msg.linear.y = 0.0
    vel_msg.linear.z = 0.0  # Mantener la altura actual si no se presiona ninguna tecla de elevación
    vel_msg.angular.z = 0.0
    vel_pub.publish(vel_msg)

def up():
    setText("Up")
    vel_msg = Twist()
    vel_msg.linear.z = float(1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def down():
    setText("Down")
    vel_msg = Twist()
    vel_msg.linear.z = float(-1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def forward():
    setText("Fordward")
    vel_msg = Twist()
    vel_msg.linear.x = float(1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def backward():
    setText("Backward")
    vel_msg = Twist()
    vel_msg.linear.x = float(-1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def right():
    setText("Right")
    vel_msg = Twist()
    vel_msg.linear.y = float(-1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def left():
    setText("Left")
    vel_msg = Twist()
    vel_msg.linear.y = float(1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def cw():
    setText("Turn Right")
    vel_msg = Twist()
    vel_msg.angular.z = float(-1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

def ccw():
    setText("Turn Left")
    vel_msg = Twist()
    vel_msg.angular.z = float(1.0)
    vel_pub.publish(vel_msg)
    rospy.sleep(0.5)
    enviar_velocidad(0.0,0.0,0.0,0.0)

#Despliegue datos de odometria y altura
ttk.Label(mainframe, textvariable=x_p).grid(column=1, row=2, sticky=(tk.W,tk.E))
ttk.Label(mainframe, textvariable=y_p).grid(column=2, row=2, sticky=(tk.W,tk.E))
ttk.Label(mainframe, textvariable=z_p).grid(column=3, row=2, sticky=(tk.W,tk.E))
ttk.Label(mainframe, textvariable=z_o).grid(column=4, row=2, sticky=(tk.W,tk.E))
ttk.Label(mainframe, text="X (m)").grid(column=1, row=3, sticky=tk.W)
ttk.Label(mainframe, text="Y (m)").grid(column=2, row=3, sticky=tk.W)
ttk.Label(mainframe, text="Z (m)").grid(column=3, row=3, sticky=tk.W)
ttk.Label(mainframe, text="Yaw (°)").grid(column=4, row=3, sticky=tk.W)

#Botones de Control
ttk.Button(mainframe, text="Turn Left", command=ccw).grid(column=1, row=5, sticky=tk.W)
ttk.Button(mainframe, text="Fordward", command=forward).grid(column=2, row=5, sticky=tk.W)
ttk.Button(mainframe, text="Turn Right", command=cw).grid(column=3, row=5, sticky=tk.W)
ttk.Button(mainframe, text="Left", command=left).grid(column=1, row=6, sticky=tk.W)
ttk.Button(mainframe, text="Hover", command=hover).grid(column=2, row=6, sticky=tk.W)
ttk.Button(mainframe, text="Right", command=right).grid(column=3, row=6, sticky=tk.W)
ttk.Button(mainframe, text="Up", command=up).grid(column=1, row=7, sticky=tk.W)
ttk.Button(mainframe, text="Backward", command=backward).grid(column=2, row=7, sticky=tk.W)
ttk.Button(mainframe, text="Down", command=down).grid(column=3, row=7, sticky=tk.W)
ttk.Button(mainframe, text="Take Off", command=takeoff).grid(column=3, row=4, sticky=tk.W)
ttk.Button(mainframe, text="Land", command=land).grid(column=2, row=4, sticky=tk.W)


ttk.Label(mainframe, text="1").grid(column=3, row=1, sticky=tk.W)

root.mainloop()
