#!/usr/bin/env python3
import rospy
from geometry_msgs.msg import Twist
from sensor_msgs.msg import LaserScan

pub = rospy.Publisher("cmd_vel", Twist, queue_size=10)
velocity_msg = Twist()

# Parámetros de la exploración
linear_velocity = 0.2  # Velocidad lineal constante
angular_velocity = 0.5  # Velocidad angular constante
obstacle_threshold = 0.5  # Distancia mínima para considerar un obstáculo
follow_wall_distance = 1.0  # Distancia a la pared para seguir
follow_wall_angle = 30.0  # Ángulo de seguimiento de pared

def callback_laser(msg):
    # Obtener los datos del escaneo láser
    laser_range = msg.ranges

    # Verificar si hay obstáculos cercanos en el rango frontal
    if min(laser_range[6:355]) < obstacle_threshold:
        # Si hay un obstáculo en el rango frontal, seguir la pared
        follow_wall(laser_range)
    else:
        # Si no hay obstáculos, avanzar recto
        velocity_msg.linear.x = linear_velocity
        velocity_msg.angular.z = 0.0

    # Publicar el mensaje de velocidad
    pub.publish(velocity_msg)

def follow_wall(laser_range):
    # Obtener el índice del rango más cercano al ángulo de seguimiento de pared
    angle_index = int(follow_wall_angle / 360.0 * len(laser_range))

    # Calcular el rango objetivo para seguir la pared
    target_range = laser_range[angle_index]

    # Calcular la diferencia entre el rango objetivo y la distancia deseada a la pared
    diff_range = target_range - follow_wall_distance

    # Calcular la velocidad angular proporcional a la diferencia de rango
    velocity_msg.linear.x = linear_velocity
    velocity_msg.angular.z = -diff_range * angular_velocity

def explore():
    # Iniciar el nodo de ROS
    rospy.init_node('exploration')

    # Suscribirse al escaneo láser
    rospy.Subscriber('/scan', LaserScan, callback_laser)

    # Mantener la exploración mientras no se interrumpa
    rospy.spin()

if __name__ == "__main__":
    try:
        explore()
    except rospy.ROSInterruptException:
        pass

